@@ -1,14 +1,16 @@
 package Mercury;
 # ABSTRACT: Main broker application class
-$Mercury::VERSION = '0.003';
+$Mercury::VERSION = '0.004';
 
 use Mojo::Base 'Mojolicious';
 use Scalar::Util qw( refaddr );
 use File::Basename qw( dirname );
 use File::Spec::Functions qw( catdir );
+use Mercury::PushPull;
 
 my %pubsub_topics;
 my %bus_topics;
+my %pushpull;
 
 
 sub add_bus_peer {
@@ -104,6 +106,24 @@ sub route_websocket_pub {
     } );
 }
 
+
+sub route_websocket_pull {
+    my ( $c ) = @_;
+    Mojo::IOLoop->stream($c->tx->connection)->timeout(1200);
+    my $topic = $c->stash( 'topic' );
+    my $pat = $pushpull{ $topic } ||= Mercury::PushPull->new( topic => $topic );
+    $pat->add_puller( $c );
+};
+
+
+sub route_websocket_push {
+    my ( $c ) = @_;
+    Mojo::IOLoop->stream($c->tx->connection)->timeout(1200);
+    my $topic = $c->stash( 'topic' );
+    my $pat = $pushpull{ $topic } ||= Mercury::PushPull->new( topic => $topic );
+    $pat->add_pusher( $c );
+}
+
 sub startup {
     my ( $app ) = @_;
     $app->plugin( 'Config', { default => { broker => { } } } );
@@ -115,17 +135,40 @@ sub startup {
     $app->helper( add_bus_peer => \&add_bus_peer );
     $app->helper( remove_bus_peer => \&remove_bus_peer );
     $app->helper( send_bus_message => \&send_bus_message );
+
     my $r = $app->routes;
+    if ( my $origin = $app->config->{broker}{allow_origin} ) {
+        # Allow only '*' for wildcards
+        my @origin = map { quotemeta } ref $origin eq 'ARRAY' ? @$origin : $origin;
+        s/\\\*/.*/g for @origin;
+
+        $r = $r->under( '/' => sub {
+            #say "Got origin: " . $_[0]->req->headers->origin;
+            #say "Checking against: @origin";
+            my $origin = $_[0]->req->headers->origin;
+            if ( !$origin || !grep { $origin =~ /$_/ } @origin ) {
+                $_[0]->render(
+                    status => '401',
+                    text => 'Origin check failed',
+                );
+                return;
+            }
+            return 1;
+        } );
+    }
+
     $r->websocket( '/sub/*topic' )->to( cb => \&route_websocket_sub )->name( 'sub' );
     $r->websocket( '/pub/*topic' )->to( cb => \&route_websocket_pub )->name( 'pub' );
     $r->websocket( '/bus/*topic' )->to( cb => \&route_websocket_bus )->name( 'bus' );
+    $r->websocket( '/push/*topic' )->to( cb => \&route_websocket_push )->name( 'push' );
+    $r->websocket( '/pull/*topic' )->to( cb => \&route_websocket_pull )->name( 'pull' );
 
     if ( $app->mode eq 'development' ) {
         # Enable the example app
         my $root = catdir( dirname( __FILE__ ), 'Mercury' );
         $app->static->paths->[0] = catdir( $root, 'public' );
         $app->renderer->paths->[0] = catdir( $root, 'templates' );
-        $r->any( '/' )->to( cb => sub { shift->render( 'index' ) } );
+        $app->routes->any( '/' )->to( cb => sub { shift->render( 'index' ) } );
     }
 }
 
@@ -141,7 +184,7 @@ Mercury - Main broker application class
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 DESCRIPTION
 
@@ -175,6 +218,16 @@ to the topic or any child topics will be sent to this subscriber.
 Establish a WebSocket to publish to the given C<topic>. Messages published to
 the topic will be sent to all subscribers to the topic or any parent topics.
 
+=head2 /pull/*topic
+
+Establish a WebSocket to pull messages from the given C<topic>. Messages will
+be routed in a round-robin fashion from pushers.
+
+=head2 /push/*topic
+
+Establish a WebSocket to push messages to the given C<topic>. Messages will be
+routed in a round-robin fashion to a single puller.
+
 =head1 METHODS
 
 =head2 add_bus_peer
