@@ -1,6 +1,6 @@
 package Mercury::Command::mercury;
 # ABSTRACT: Mercury command for Mojolicious
-$Mercury::Command::mercury::VERSION = '0.003';
+$Mercury::Command::mercury::VERSION = '0.004';
 use Mojo::Base 'Mojolicious::Commands';
 
 has description => 'Mercury message broker';
@@ -27,7 +27,7 @@ Mercury::Command::mercury - Mercury command for Mojolicious
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 SYNOPSIS
 
@@ -43,7 +43,7 @@ L<Mercury::Command::mercury> lists available L<Mercury> commands.
 
 =item *
 
-L<Mercury>
+L<mercury>
 
 =back
 
