@@ -1,6 +1,6 @@
 package Mercury::Command::mercury::broker;
 # ABSTRACT: Mercury message broker command
-$Mercury::Command::mercury::broker::VERSION = '0.003';
+$Mercury::Command::mercury::broker::VERSION = '0.004';
 
 use Mojo::Base 'Mojolicious::Command';
 use Mercury;
@@ -60,7 +60,7 @@ Mercury::Command::mercury::broker - Mercury message broker command
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 SYNOPSIS
 
@@ -122,7 +122,7 @@ Run this command.
 
 =item *
 
-L<Mercury>
+L<mercury>
 
 =back
 
